// ignore_for_file: file_names

import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:http/http.dart' as http;
import 'package:meet_hour/types/view_meeting_type.dart';

import 'constants/constants.dart';
import 'types/add_contact_type.dart';
import 'types/contacts_type.dart';
import 'types/edit_contact_type.dart';
import 'types/edit_meeting_type.dart';
import 'types/generate_jwt_type.dart';
import 'types/login_type.dart';
import 'types/recordings_list.dart';
import 'types/refresh_token_type.dart';
import 'types/schedule_meeting_type.dart';

enum ApiEndpoint {
  addContact,
  login,
  scheduleMeeting,
  getJwt,
  upcomingMeeting,
  timezone,
  recordingsList,
  editMeeting,
  editContact,
  viewMeeting,
  archiveMeeting,
  missedMeetings,
  completedMeetings,
  userDetails,
  refreshToken,
  contactsList,
}


String apiEndpointUrl(String baseUrl, ApiEndpoint endpoint) {
  switch (endpoint) {
    case ApiEndpoint.login:
      return '$baseUrl/oauth/token';
    case ApiEndpoint.addContact:
      return '$baseUrl/api/$API_VERSION/customer/addcontact';
    case ApiEndpoint.contactsList:
      return '$baseUrl/api/$API_VERSION/customer/contacts';
    case ApiEndpoint.scheduleMeeting:
      return '$baseUrl/api/$API_VERSION/meeting/schedulemeeting';
    case ApiEndpoint.getJwt:
      return '$baseUrl/api/$API_VERSION/getjwt';
    case ApiEndpoint.upcomingMeeting:
      return '$baseUrl/api/$API_VERSION/meeting/upcomingmeetings';
    case ApiEndpoint.timezone:
      return '$baseUrl/api/$API_VERSION/getTimezone';
    case ApiEndpoint.recordingsList:
      return '$baseUrl/api/$API_VERSION/customer/videorecordinglist';
    case ApiEndpoint.editMeeting:
      return '$baseUrl/api/$API_VERSION/meeting/editmeeting';
    case ApiEndpoint.editContact:
      return '$baseUrl/api/$API_VERSION/customer/editcontact';
    case ApiEndpoint.viewMeeting:
      return '$baseUrl/api/$API_VERSION/meeting/viewmeeting';
    case ApiEndpoint.archiveMeeting:
        return '$baseUrl/api/$API_VERSION/meeting/archivemeeting';
    case ApiEndpoint.missedMeetings:
        return '$baseUrl/api/$API_VERSION/meeting/missedmeetings';
    case ApiEndpoint.completedMeetings:
        return '$baseUrl/api/$API_VERSION/meeting/completedmeeting';
    case  ApiEndpoint.userDetails:
        return '$baseUrl/api/$API_VERSION/customer/user_details';
    case ApiEndpoint.refreshToken:
        return '$baseUrl/oauth/token';
  }
}


// ignore: avoid_classes_with_only_static_members
class ApiServices {
  static final String BASE_URL = 'https://api.meethour.io';


static dynamic getHeaders(String token) {
    return <String, String>{
      'Accept': 'application/json',
      'Content-type': 'application/json',
      'Authorization': 'Bearer $token'
    };
  }


static dynamic postFetch(String token, ApiEndpoint endpoint, dynamic body) async {
    final String constructedUrl = apiEndpointUrl(BASE_URL, endpoint);
try {
  var url = Uri.parse(constructedUrl);
  var response = await http.post(url, body: jsonEncode(body), headers: getHeaders(token));
  if (response.statusCode >= 400 && response.statusCode < 600) {
    if (kDebugMode) {
      print('Bad response from server.');
    }
    return null;
  }
  return json.decode(response.body);
} catch (err) {
  if (kDebugMode) {
    print(err);
  }
}

}


static dynamic login(LoginType body) async {
    return postFetch('', ApiEndpoint.login, body);
  }


static dynamic getRefreshToken(String token, RefreshTokenType body) async {
    return postFetch(token, ApiEndpoint.refreshToken, body);
  }

  static dynamic userDetails(String token) async {
    return postFetch(token, ApiEndpoint.userDetails, '');
  }


static dynamic generateJwt(String token, GenerateJwtType body) async {
    return postFetch(token, ApiEndpoint.getJwt, body);
  }


static dynamic addContact(String token, AddContactType body) async {
    return postFetch(token, ApiEndpoint.addContact, body);
  }


static dynamic timezone(String token) async {
    return postFetch(token, ApiEndpoint.timezone, {});
  }


static dynamic contactsList(String token, ContactsType body) async {
    return postFetch(token, ApiEndpoint.contactsList, body);
  }

 static dynamic editContact(String token, EditContactType body) async {
    return postFetch(token, ApiEndpoint.editContact, body);
  }


static dynamic scheduleMeeting(String token, ScheduleMeetingType body) async {
    return postFetch(token, ApiEndpoint.scheduleMeeting, body);
  }


static dynamic upcomingMeetings(String token, dynamic body) async {
    return postFetch(token, ApiEndpoint.upcomingMeeting, body);
  }


static dynamic archiveMeeting(String token, dynamic body) async {
    return postFetch(token, ApiEndpoint.archiveMeeting, body);
  }


static dynamic missedMeetings(String token, dynamic body) async {
    return postFetch(token, ApiEndpoint.missedMeetings, body);
  }

static dynamic completedMeetings(String token, dynamic body) async {
    return postFetch(token, ApiEndpoint.completedMeetings, body);
  }


static dynamic editMeeting(String token, EditMeetingType body) async {
    return postFetch(token, ApiEndpoint.editMeeting, body);
  }


static dynamic viewMeeting(String token, ViewMeetingType body) async {
    return postFetch(token, ApiEndpoint.viewMeeting, body);
  }


static dynamic recordingsList(String token, RecordingsListType body) async {
    return postFetch(token, ApiEndpoint.recordingsList, body);
  }

}
